package eMonMusical.flow;

import javax.servlet.http.HttpServletRequest;

import org.springframework.webflow.context.servlet.ServletExternalContext;
import org.springframework.webflow.execution.Action;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

import eMonMusical.domini.Compra;
import eMonMusical.domini.Preferits;
import eMonMusical.domini.Producte;
import eMonMusical.domini.ProducteComprat;
import eMonMusical.domini.Usuari;
import eMonMusical.domini.Vendes;
import eMonMusical.exceptions.QuantitatNegativaException;
import eMonMusical.exceptions.ProducteQuantitatException;
import eMonMusical.exceptions.QuantitatZeroException;
import eMonMusical.servici.GestorPreferits;
import eMonMusical.servici.GestorProducte;
import eMonMusical.servici.GestorVendes;

public class MostrarProductesCompratsAction implements Action {
	
	private GestorProducte gestorProducte;
	private GestorPreferits gestorPreferits;
	private GestorVendes gestorVendes;

	@Override
	public Event execute(RequestContext context) throws Exception {
		
		ServletExternalContext externalContext = (ServletExternalContext) context.getExternalContext();
		HttpServletRequest request = (HttpServletRequest) externalContext.getRequest();
		String nUsuari = (String) request.getSession().getAttribute("nUsuari");

		String[] codis = context.getRequestParameters().getArray("prod.codi");
		String[] quantitats = context.getRequestParameters().getArray("prod.quantitat");
		Compra compra = (Compra) context.getFlowScope().get("compra2");
		Usuari usuari = new Usuari();
		usuari.setnUsuari(nUsuari);
		compra.setUsuari(usuari);
		
		int size = codis.length;
		int i = 0;
		float granTotal = 0;
		
		while(size>0){
			String codi = codis[i];
			int quantitat = new Integer(quantitats[i]);
			Producte prod = gestorProducte.getProducte(codi);
			if(quantitat > prod.getQuantitat()){
				compra.afegirProducte(prod);
				throw new ProducteQuantitatException();
			}else if(quantitat == 0){
				compra.afegirProducte(prod);
				throw new QuantitatZeroException();
			}else if(quantitat < 0){
				compra.afegirProducte(prod);
				throw new QuantitatNegativaException();
			}
			size--;
			i++;
		}
		
		size = codis.length;
		i = 0;
		
		while(size>0){
			String codi = codis[i];
			int quantitat = new Integer(quantitats[i]);
			Producte prod = gestorProducte.getProducte(codi);
			ProducteComprat pc = new ProducteComprat();
			pc.setCodi(codi);
			pc.setTitol(prod.getTitol());
			pc.setPreu(prod.getPreu());
			pc.setQuantitat(quantitat);
			float preu = pc.getPreu();
			int pcquantitat = pc.getQuantitat();
			float total = preu * pcquantitat;
			pc.setTotal(total);
			granTotal = granTotal + pc.getTotal();
			compra.afegirComprat(pc);
			Preferits pref = new Preferits();
			pref.setCodiProducte(codi);
			pref.setnUsuari(compra.getUsuari().getnUsuari());
			pref.setQuantitat(quantitat);
			gestorPreferits.guardarPreferit(pref);
			Vendes vendes = new Vendes();
			vendes.setCodi(pref.getCodiProducte());
			vendes.setnUsuari(pref.getnUsuari());
			vendes.setPreu(prod.getPreu());
			vendes.setQuantitat(quantitat);
			gestorVendes.guardarVenda(vendes);
			size--;
			i++;
		}
		
		compra.setGranTotal(granTotal);
		
		
		return new Event(this, "success");
	}

	public void setGestorProducte(GestorProducte gestorProducte) {
		this.gestorProducte = gestorProducte;
	}

	public void setGestorPreferits(GestorPreferits gestorPreferits) {
		this.gestorPreferits = gestorPreferits;
	}

	public void setGestorVendes(GestorVendes gestorVendes) {
		this.gestorVendes = gestorVendes;
	}

	public GestorVendes getGestorVendes() {
		return gestorVendes;
	}
	
	
	

}
