package eMonMusical.repositori;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;

import eMonMusical.domini.Preferits;
import eMonMusical.domini.ProducteComprat;
import eMonMusical.domini.ProductePreferit;
import eMonMusical.servici.GestorProducte;

public class JdbcPreferitsDao extends SimpleJdbcDaoSupport implements
		PreferitsDao {

	protected final Log logger = LogFactory.getLog(getClass());

	private GestorProducte gestorProducte;

	public GestorProducte getGestorProducte() {
		return gestorProducte;
	}

	public void setGestorProducte(GestorProducte gestorProducte) {
		this.gestorProducte = gestorProducte;
	}

	@Override
	public void guardarPreferit(Preferits preferit) {
		logger.info("Actualitzant preferits d'usuari " + preferit.getnUsuari()
				+ " i producte amb codi " + preferit.getCodiProducte());
		Calendar cal = Calendar.getInstance(TimeZone.getDefault());
		java.util.Date avui = cal.getTime();

		int quantitat = getJdbcTemplate().queryForInt(
				"SELECT " + "taula_producte.quantitat_disponible "
						+ "FROM public.taula_producte "
						+ "WHERE taula_producte.codi_producte = '"
						+ preferit.getCodiProducte() + "';");

		int disponible = quantitat - preferit.getQuantitat();

		getJdbcTemplate().update(
				"UPDATE taula_producte " + "SET quantitat_disponible="
						+ disponible + " WHERE taula_producte.codi_producte='"
						+ preferit.getCodiProducte() + "';");

		getSimpleJdbcTemplate().update(
				"INSERT INTO taula_preferits(nom_usuari, "
						+ "codi_producte, quantitat, data_preferit) "
						+ "VALUES (?, ?, ?, ?);", preferit.getnUsuari(),
				preferit.getCodiProducte(), preferit.getQuantitat(), avui);

	}

	@Override
	public List<ProductePreferit> mostrarPreferits() {
		logger.info("Mirant preferits.");
		List<ProductePreferit> preferits = getSimpleJdbcTemplate()
				.query("SELECT taula_preferits.*, "
						+ "taula_producte.titol "
						+ "FROM taula_preferits, taula_producte "
						+ "WHERE taula_preferits.codi_producte = taula_producte.codi_producte "
						+ "Group by taula_preferits.nom_usuari, "
						+ "taula_preferits.codi_producte, taula_producte.titol, "
						+ "taula_preferits.quantitat, taula_preferits.data_preferit "
						+ "ORDER BY nom_usuari asc;", new AdminPrefMapper());
		return preferits;
	}

	@SuppressWarnings({ "rawtypes" })
	@Override
	public List<ProducteComprat> mostrarCistellaCompra(String nUsuari) {
		logger.info("Mirant la cistella de compra de l'usuari " + nUsuari + ".");
		Calendar cal = Calendar.getInstance(TimeZone.getDefault());
		java.util.Date avui = cal.getTime();

		List<ProducteComprat> comprats = getSimpleJdbcTemplate()
				.query("SELECT taula_preferits.codi_producte, taula_preferits.quantitat, "
						+ "taula_producte.titol, taula_producte.preu "
						+ "FROM public.taula_preferits, public.taula_producte "
						+ "WHERE taula_preferits.nom_usuari = '"
						+ nUsuari
						+ "' "
						+ "AND taula_preferits.data_preferit = '"
						+ avui
						+ "' "
						+ "AND taula_preferits.codi_producte = taula_producte.codi_producte;",
						new CompratsMapper());

		List codis = getJdbcTemplate()
				.queryForList(
						"SELECT DISTINCT taula_preferits.codi_producte "
								+ "FROM public.taula_preferits, public.taula_producte "
								+ "WHERE taula_preferits.nom_usuari = '"
								+ nUsuari
								+ "' "
								+ "AND taula_preferits.data_preferit = '"
								+ avui
								+ "' "
								+ "AND taula_preferits.codi_producte = taula_producte.codi_producte;",
						String.class);

		List<ProducteComprat> cistellaCompra = new ArrayList<ProducteComprat>();

		int sItems = codis.size();
		int i = 0;

		while (sItems > 0) {
			String prodCodi = (String) codis.get(i);
			ProducteComprat prod = new ProducteComprat();
			prod.setCodi(prodCodi);
			int sComprats = comprats.size();
			int j = 0;
			int quantitat = 0;
			while (sComprats > 0) {
				ProducteComprat pc = comprats.get(j);
				String pcCodi = pc.getCodi();
				if (pcCodi.equals(prodCodi)) {
					quantitat = quantitat + pc.getQuantitat();
					prod.setTitol(pc.getTitol());
				}
				sComprats--;
				j++;
			}
			prod.setQuantitat(quantitat);
			cistellaCompra.add(prod);
			sItems--;
			i++;
		}

		return cistellaCompra;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public List<ProducteComprat> mostrarPreferitsUsuari(String nUsuari) {

		logger.info("Mirant els preferits de l'usuari " + nUsuari + ".");

		List<ProducteComprat> comprats = getSimpleJdbcTemplate()
				.query("SELECT taula_preferits.codi_producte, taula_preferits.quantitat, "
						+ "taula_producte.titol, taula_producte.preu "
						+ "FROM public.taula_preferits, public.taula_producte "
						+ "WHERE taula_preferits.nom_usuari = '"
						+ nUsuari
						+ "' "
						+ "AND taula_preferits.codi_producte = taula_producte.codi_producte;",
						new CompratsMapper());

		List codis = getJdbcTemplate()
				.queryForList(
						"SELECT DISTINCT taula_preferits.codi_producte "
								+ "FROM public.taula_preferits, public.taula_producte "
								+ "WHERE taula_preferits.nom_usuari = '"
								+ nUsuari
								+ "' "
								+ "AND taula_preferits.codi_producte = taula_producte.codi_producte;",
						String.class);

		List<ProducteComprat> preferits = new ArrayList<ProducteComprat>();

		int sItems = codis.size();
		int i = 0;

		while (sItems > 0) {
			String prodCodi = (String) codis.get(i);
			ProducteComprat prod = new ProducteComprat();
			prod.setCodi(prodCodi);
			int sComprats = comprats.size();
			int j = 0;
			int quantitat = 0;
			while (sComprats > 0) {
				ProducteComprat pc = comprats.get(j);
				String pcCodi = pc.getCodi();
				if (pcCodi.equals(prodCodi)) {
					quantitat = quantitat + pc.getQuantitat();
					prod.setTitol(pc.getTitol());
				}
				sComprats--;
				j++;
			}
			prod.setQuantitat(quantitat);
			preferits.add(prod);
			sItems--;
			i++;
		}

		return preferits;

	}

	@SuppressWarnings("unused")
	private static class PreferitsMapper implements
			ParameterizedRowMapper<Preferits> {

		public Preferits mapRow(ResultSet rs, int numFila) throws SQLException {
			Preferits pref = new Preferits();
			pref.setCodiProducte(rs.getString("codi_producte"));
			pref.setDataPreferit(rs.getString("data_preferit"));
			pref.setnUsuari(rs.getString("nom_usuari"));
			pref.setQuantitat(rs.getInt("quantitat"));
			return pref;
		}

	}

	private static class AdminPrefMapper implements
			ParameterizedRowMapper<ProductePreferit> {

		public ProductePreferit mapRow(ResultSet rs, int numFila)
				throws SQLException {
			ProductePreferit pref = new ProductePreferit();
			pref.setCodiProducte(rs.getString("codi_producte"));
			try {
				String data = rs.getString("data_preferit");
				Date dataNo = new SimpleDateFormat("yyyy-MM-dd").parse(data);
				String dataSi = new SimpleDateFormat("dd-MM-yyyy")
						.format(dataNo);
				pref.setDataPreferit(dataSi);
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			pref.setnUsuari(rs.getString("nom_usuari"));
			pref.setQuantitat(rs.getInt("quantitat"));
			pref.setTitol(rs.getString("titol"));
			return pref;
		}

	}

	private static class CompratsMapper implements
			ParameterizedRowMapper<ProducteComprat> {

		public ProducteComprat mapRow(ResultSet rs, int numFila)
				throws SQLException {
			ProducteComprat prod = new ProducteComprat();
			prod.setCodi(rs.getString("codi_producte"));
			prod.setPreu(rs.getFloat("preu"));
			prod.setQuantitat(rs.getInt("quantitat"));
			prod.setTitol(rs.getString("titol"));
			return prod;
		}

	}

}
