package eMonMusical.repositori;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;

import eMonMusical.domini.Producte;
import eMonMusical.domini.Votacio;

public class JdbcProducteDao extends SimpleJdbcDaoSupport implements
		ProducteDao {

	protected final Log logger = LogFactory.getLog(getClass());

	@Override
	public List<Producte> getProducteVotsList() {
		logger.info("Mirant productes vots!");
		List<Producte> productes = getSimpleJdbcTemplate()
				.query("SELECT taula_producte.codi_producte, titol, preu, descompte, cantant, quantitat_disponible, "
						+ "tipus, data_publicacio, nombre_vots FROM taula_producte, taula_votacio "
						+ "WHERE taula_producte.codi_producte=taula_votacio.codi_producte ORDER BY nombre_vots desc;",
						new ProducteMapper());
		return productes;
	}

	@Override
	public List<Producte> getProductesTipusList(String tipus) {
		logger.info("Mirant producte tipus seleccionat!");
		List<Producte> productes = getSimpleJdbcTemplate()
				.query("SELECT DISTINCT taula_producte.codi_producte, titol, preu, descompte, cantant, quantitat_disponible, "
						+ "tipus, data_publicacio, nombre_vots FROM taula_producte, taula_votacio "
						+ "WHERE taula_producte.tipus="
						+ "'"
						+ tipus
						+ "' AND "
						+ "taula_producte.codi_producte=taula_votacio.codi_producte"
						+ " ORDER BY codi_producte;", new ProducteMapper());
		return productes;
	}

	@Override
	public List<Producte> getProductesTitolList(String titol) {
		logger.info("Mirant producte titol seleccionat!");
		List<Producte> productes = getSimpleJdbcTemplate()
				.query("SELECT DISTINCT taula_producte.codi_producte, titol, preu, descompte, cantant, quantitat_disponible, "
						+ "tipus, data_publicacio, nombre_vots FROM taula_producte, taula_votacio "
						+ "WHERE taula_producte.titol="
						+ "'"
						+ titol
						+ "' AND "
						+ "taula_producte.codi_producte=taula_votacio.codi_producte"
						+ " ORDER BY codi_producte;", new ProducteMapper());
		return productes;
	}

	@Override
	public List<Producte> getProductes() {
		logger.info("Mirant productes!");
		List<Producte> productes = getSimpleJdbcTemplate()
				.query("SELECT taula_producte.codi_producte, titol, preu, descompte, cantant, quantitat_disponible, "
						+ "tipus, data_publicacio, nombre_vots FROM taula_producte, taula_votacio "
						+ "WHERE taula_producte.codi_producte=taula_votacio.codi_producte ORDER BY nombre_vots desc;",
						new ProducteMapper());
		return productes;
	}

	@Override
	public List<Producte> getProducteNovetats() {
		logger.info("Mirant novetats!");
		List<Producte> productes = getSimpleJdbcTemplate()
				.query("SELECT taula_producte.codi_producte, titol, preu, descompte, cantant, quantitat_disponible, "
						+ "tipus, data_publicacio, nombre_vots FROM taula_producte, taula_votacio "
						+ "WHERE taula_producte.codi_producte=taula_votacio.codi_producte AND datediff('m', "
						+ "data_publicacio, current_date)<=1 Order by data_publicacio desc;",
						new ProducteMapper());
		return productes;
	}

	@Override
	public Producte getProducte(String codi) {
		logger.info("Buscant producte amb codi: " + codi);
		Producte prod = getSimpleJdbcTemplate()
				.queryForObject(
						"SELECT taula_producte.codi_producte, taula_producte.titol, "
								+ "taula_producte.preu, taula_producte.descompte, "
								+ "taula_producte.cantant, taula_producte.quantitat_disponible, "
								+ "taula_producte.tipus, taula_producte.data_publicacio, "
								+ "taula_votacio.nombre_vots "
								+ "FROM taula_producte, taula_votacio "
								+ "WHERE taula_producte.codi_producte='"
								+ codi
								+ "' AND taula_votacio.codi_producte=taula_producte.codi_producte;",
						new ProducteMapper());
		return prod;
	}

	@Override
	public void actualitzarProducte(Producte prod) throws ParseException {
		logger.info("Actualitzant producte: " + prod.getTitol());
		String data = prod.getData();
		SimpleDateFormat formateador = new SimpleDateFormat("dd-MM-yyyy");
		java.util.Date publica = formateador.parse(data);
		int comptar = getSimpleJdbcTemplate()
				.update("UPDATE taula_producte SET preu= :preu, descompte= :descompte,"
						+ " quantitat_disponible= :quantitat_disponible, titol= :titol,"
						+ "cantant= :cantant, tipus= :tipus, data_publicacio= :data_publicacio "
						+ "WHERE codi_producte= :codi_producte;",
						new MapSqlParameterSource()
								.addValue("titol", prod.getTitol())
								.addValue("preu", prod.getPreu())
								.addValue("descompte", prod.getDescompte())
								.addValue("quantitat_disponible",
										prod.getQuantitat())
								.addValue("codi_producte", prod.getCodi())
								.addValue("tipus", prod.getTipus())
								.addValue("cantant", prod.getCantant())
								.addValue("data_publicacio", publica));
		logger.info("Files afectades: " + comptar);

	}

	@Override
	public void afegirProducte(Producte prod) throws ParseException {
		logger.info("Guardant producte: " + prod.getTitol());
		String data = prod.getData();
		SimpleDateFormat formateador = new SimpleDateFormat("dd-MM-yyyy");
		java.util.Date publica = formateador.parse(data);

		getSimpleJdbcTemplate().update(
				"INSERT INTO " + "taula_producte(codi_producte, titol, preu, "
						+ "descompte, cantant, quantitat_disponible, "
						+ "tipus, data_publicacio) "
						+ "VALUES (?, ?, ?, ?, ?, ?, ?, ?);", prod.getCodi(),
				prod.getTitol(), prod.getPreu(), prod.getDescompte(),
				prod.getCantant(), prod.getQuantitat(), prod.getTipus(),
				publica);

		getSimpleJdbcTemplate().update(
				"INSERT INTO " + "taula_votacio(codi_producte, nombre_vots) "
						+ "VALUES (?, ?);", prod.getCodi(), 0);

	}

	@Override
	public Producte getProductesVenuts(String codi) {
		Producte prod = new Producte();

		prod = getSimpleJdbcTemplate()
				.queryForObject(
						"SELECT taula_vendes.*, taula_producte.codi_producte, taula_producte.titol, "
								+ "taula_producte.preu, taula_producte.descompte, "
								+ "taula_producte.cantant, taula_producte.quantitat_disponible, "
								+ "taula_producte.tipus, taula_producte.data_publicacio, "
								+ "taula_votacio.nombre_vots "
								+ "FROM taula_producte, taula_votacio "
								+ "WHERE taula_producte.codi_producte='"
								+ codi
								+ "' AND taula_votacio.codi_producte=taula_producte.codi_producte;",
						new ProducteMapper());
		return prod;
	}

	private static class ProducteMapper implements
			ParameterizedRowMapper<Producte> {

		public Producte mapRow(ResultSet rs, int numFila) throws SQLException {
			Producte prod = new Producte();
			Votacio vot = new Votacio();
			prod.setCodi(rs.getString("codi_producte"));
			prod.setDescompte(rs.getString("descompte"));
			prod.setPreu(rs.getFloat("preu"));
			prod.setQuantitat(rs.getInt("quantitat_disponible"));
			prod.setCantant(rs.getString("cantant"));
			try {
				String data = rs.getString("data_publicacio");
				Date dataNo = new SimpleDateFormat("yyyy-MM-dd").parse(data);
				String dataSi = new SimpleDateFormat("dd-MM-yyyy")
						.format(dataNo);
				prod.setData(dataSi);
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			prod.setTitol(rs.getString("titol"));
			prod.setTipus(rs.getString("tipus"));
			vot.setCodi(rs.getString("codi_producte"));
			vot.setNombreVots(rs.getInt("nombre_vots"));
			prod.setVotacio(vot);
			return prod;
		}

	}

	@Override
	public List<Votacio> getVotacioList() {
		logger.info("Mirant vots!");
		List<Votacio> vots = getSimpleJdbcTemplate().query(
				"SELECT codi_producte, nombre_vots FROM taula_votacio;",
				new VotacioMapper());
		return vots;
	}

	public List<Votacio> getVotacioCodiList(String codi) {
		logger.info("Mirant votacio codi seleccionat!");
		List<Votacio> vots = getSimpleJdbcTemplate().query(
				"SELECT taula_votacio.codi_producte, nombre_vots FROM taula_votacio "
						+ "WHERE taula_votacio.codi_producte=" + "'" + codi
						+ "' ORDER BY codi_producte;", new VotacioMapper());
		return vots;
	}

	@Override
	public void guardarVotacio(Votacio vot) {
		logger.info("Guardant vot del producte amb codi: " + vot.getCodi());
		int comptar = getSimpleJdbcTemplate()
				.update("UPDATE taula_votacio SET nombre_vots= :nombre_vots WHERE codi_producte= :codi_producte;",
						new MapSqlParameterSource().addValue("codi_producte",
								vot.getCodi()).addValue("nombre_vots",
								vot.getNombreVots()));
		logger.info("Columnes afectades: " + comptar);

	}

	private static class VotacioMapper implements
			ParameterizedRowMapper<Votacio> {

		public Votacio mapRow(ResultSet rs, int numFila) throws SQLException {
			Votacio vot = new Votacio();
			vot.setCodi(rs.getString("codi_producte"));
			vot.setNombreVots(rs.getInt("nombre_vots"));
			return vot;
		}

	}

}
