package eMonMusical.repositori;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;

import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;
import org.springframework.transaction.annotation.Transactional;

import eMonMusical.domini.IniciSessioInfo;
import eMonMusical.domini.Usuari;

public class JdbcUsuariDao extends SimpleJdbcDaoSupport implements UsuariDao {

	@Override
	public List<Usuari> getUsuaris() {
		logger.info("Mirant usuaris!");
		List<Usuari> usuaris = getSimpleJdbcTemplate()
				.query("SELECT codi_usuari, nom_usuari, nom, primer_cognom, "
						+ "segon_cognom, data_naixement, e_mail, adreca1, adreca2, "
						+ "ciutat, provincia, targeta_credit, tipus_targeta_credit "
						+ "FROM taula_usuari;", new UsuariMapper());
		return usuaris;
	}

	private int getCodiUsuari() {
		int codiUsuari = 0;
		int temp = 0;
		temp = getJdbcTemplate().queryForInt(
				"Select max(codi_usuari) from taula_usuari;");
		codiUsuari = temp + 1;
		return codiUsuari;
	}

	@Override
	public void guardarUsuari(Usuari user) throws ParseException {
		logger.info("Guardant usuari: " + user.getnUsuari());
		String data = user.getdNaixement();
		SimpleDateFormat formateador = new SimpleDateFormat("dd/MM/yyyy");
		java.util.Date neix = formateador.parse(data);
		int codi = this.getCodiUsuari();
		user.setCodi(codi);
		user.getIsInfo().setnUsuari(user.getnUsuari());

		getSimpleJdbcTemplate()
				.update("INSERT INTO taula_usuari(codi_usuari, nom_usuari, "
						+ "nom, primer_cognom, segon_cognom, data_naixement, e_mail, adreca1, adreca2, "
						+ "ciutat, provincia, targeta_credit, tipus_targeta_credit) "
						+ "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?);",
						user.getCodi(), user.getnUsuari(), user.getNom(),
						user.getpCognom(), user.getsCognom(), neix,
						user.geteMail(), user.getpAdre(), user.getsAdre(),
						user.getCiutat(), user.getProvincia(),
						user.getnTcredit(), user.getTipusTcredit());

		getSimpleJdbcTemplate().update(
				"INSERT INTO taula_inici_sessio_info(nom_usuari, contrasenya, "
						+ "pregunta_secreta, resposta_secreta, rol) "
						+ "VALUES (?,?,?,?,?);", user.getIsInfo().getnUsuari(),
				user.getIsInfo().getContrasenya(),
				user.getIsInfo().getPreguntaSecreta(),
				user.getIsInfo().getRespostaSecreta(), "usuari");

	}

	@Override
	public Usuari getUsuari(String nUser, String contrasenya)
			throws DataAccessException {
		return getSimpleJdbcTemplate()
				.queryForObject(
						"Select "
								+ "taula_inici_sessio_info.nom_usuari as nom_usuari, "
								+ "taula_usuari.codi_usuari, taula_usuari.nom, "
								+ "taula_usuari.primer_cognom, taula_usuari.segon_cognom, "
								+ "taula_usuari.data_naixement, taula_usuari.e_mail, "
								+ "taula_usuari.adreca1, taula_usuari.adreca2, "
								+ "taula_usuari.ciutat, taula_usuari.provincia, "
								+ "taula_usuari.targeta_credit, "
								+ "taula_usuari.tipus_targeta_credit from taula_usuari, "
								+ "taula_inici_sessio_info where taula_usuari.nom_usuari='"
								+ nUser
								+ "' and taula_inici_sessio_info.contrasenya='"
								+ contrasenya
								+ "' and taula_inici_sessio_info.nom_usuari=taula_usuari.nom_usuari;",
						new UsuariMapper());

	}

	@Override
	public IniciSessioInfo getUsuari(String nUsuari) throws DataAccessException {
		return getSimpleJdbcTemplate().queryForObject(
				"select taula_inici_sessio_info.nom_usuari, "
						+ "taula_inici_sessio_info.contrasenya, "
						+ "taula_inici_sessio_info.pregunta_secreta, "
						+ "taula_inici_sessio_info.resposta_secreta, "
						+ "taula_inici_sessio_info.rol "
						+ "from taula_inici_sessio_info "
						+ "where taula_inici_sessio_info.nom_usuari='"
						+ nUsuari + "';", new IniciSessioInfoMapper());
	}

	@Override
	public IniciSessioInfo getAdmin(String nUsuari, String contrasenya) {
		return getSimpleJdbcTemplate().queryForObject(
				"select taula_inici_sessio_info.nom_usuari, "
						+ "taula_inici_sessio_info.contrasenya, "
						+ "taula_inici_sessio_info.pregunta_secreta, "
						+ "taula_inici_sessio_info.resposta_secreta, "
						+ "taula_inici_sessio_info.rol "
						+ "from taula_inici_sessio_info "
						+ "where taula_inici_sessio_info.nom_usuari='"
						+ nUsuari
						+ "' and taula_inici_sessio_info.contrasenya='"
						+ contrasenya + "';", new IniciSessioInfoMapper());
	}
	
	@Transactional (readOnly=false)
	public void esborrarUsuari(String nUsuari) {
		getSimpleJdbcTemplate().update("DELETE FROM taula_usuari " +
				"WHERE taula_usuari.nom_usuari='"
				+ nUsuari + "';");
		
		getSimpleJdbcTemplate().update("DELETE FROM taula_inici_sessio_info " +
				"WHERE taula_inici_sessio_info.nom_usuari='"
				+ nUsuari + "';");
		
	}

	private static class UsuariMapper implements ParameterizedRowMapper<Usuari> {

		public Usuari mapRow(ResultSet rs, int numFila) throws SQLException {
			Usuari user = new Usuari();
			user.setCodi(rs.getInt("codi_usuari"));
			user.setnUsuari(rs.getString("nom_usuari"));
			user.setNom(rs.getString("nom"));
			user.setpCognom(rs.getString("primer_cognom"));
			user.setsCognom(rs.getString("segon_cognom"));
			user.setdNaixement(rs.getString("data_naixement"));
			user.seteMail(rs.getString("e_mail"));
			user.setpAdre(rs.getString("adreca1"));
			user.setsAdre(rs.getString("adreca2"));
			user.setCiutat(rs.getString("ciutat"));
			user.setProvincia(rs.getString("provincia"));
			user.setnTcredit(rs.getString("targeta_credit"));
			user.setTipusTcredit(rs.getString("tipus_targeta_credit"));
			return user;
		}

	}

	private static class IniciSessioInfoMapper implements
			ParameterizedRowMapper<IniciSessioInfo> {

		public IniciSessioInfo mapRow(ResultSet rs, int numFila)
				throws SQLException {
			IniciSessioInfo isi = new IniciSessioInfo();
			isi.setContrasenya(rs.getString("contrasenya"));
			isi.setnUsuari(rs.getString("nom_usuari"));
			isi.setPreguntaSecreta(rs.getString("pregunta_secreta"));
			isi.setRespostaSecreta(rs.getString("resposta_secreta"));
			isi.setRol(rs.getString("rol"));
			return isi;
		}

	}


}
