package eMonMusical.repositori;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;

import eMonMusical.domini.ProducteVenut;
import eMonMusical.domini.Vendes;
import eMonMusical.servici.GestorVendes;

public class JdbcVendesDao extends SimpleJdbcDaoSupport implements VendesDao {

	protected final Log logger = LogFactory.getLog(getClass());

	private GestorVendes gestorVendes;

	@Override
	public void guardarVenda(Vendes venda) {
		logger.info("Actualitzant vendes a usuari " + venda.getnUsuari()
				+ " i producte amb codi " + venda.getCodi());
		Calendar cal = Calendar.getInstance(TimeZone.getDefault());
		java.util.Date avui = cal.getTime();

		getSimpleJdbcTemplate().update(
				"INSERT INTO taula_vendes(nom_usuari, "
						+ "codi_producte, quantitat_venda, data_venda, preu) "
						+ "VALUES (?, ?, ?, ?, ?);", venda.getnUsuari(),
				venda.getCodi(), venda.getQuantitat(), avui, venda.getPreu());
	}

	@Override
	public List<ProducteVenut> mostrarVendesPerData(String data) {
		logger.info("Mirant les vendes durant el dia " + data + ".");

		List<ProducteVenut> productes = getSimpleJdbcTemplate().query(
				"SELECT taula_vendes. *, taula_producte.titol "
						+ "FROM taula_vendes, taula_producte "
						+ "WHERE taula_vendes.data_venda='" + data + "' "
						+ "AND taula_vendes.codi_producte="
						+ "taula_producte.codi_producte;",
				new ProducteVenutMapper());
		return productes;
	}

	@Override
	public List<ProducteVenut> mostrarVendesSetmanaPassada() {

		List<ProducteVenut> productes = getSimpleJdbcTemplate()
				.query("SELECT taula_vendes. *, taula_producte.titol "
						+ "FROM taula_vendes, taula_producte "
						+ "WHERE TRUNC(DATE_PART('day', now()-taula_vendes.data_venda)/7)<1 "
						+ "AND taula_vendes.codi_producte=taula_producte.codi_producte;",
						new ProducteVenutMapper());

		return productes;
	}

	@Override
	public List<ProducteVenut> mostrarVendesMesPassat() {

		List<ProducteVenut> productes = getSimpleJdbcTemplate()
				.query("SELECT taula_vendes. *, taula_producte.titol "
						+ "FROM taula_vendes, taula_producte "
						+ "WHERE datediff('m', taula_vendes.data_venda, current_date)=1 "
						+ "AND taula_vendes.codi_producte=taula_producte.codi_producte;",
						new ProducteVenutMapper());

		return productes;
	}

	public List<Vendes> mostrarVendes() {

		List<Vendes> vendes = getSimpleJdbcTemplate().query(
				"SELECT *" + "FROM taula_vendes;", new VendesMapper());

		return vendes;
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<String> mostrarDates() {
		List<String> dates = new ArrayList<String>();
			
		dates =	getJdbcTemplate().query(
				"SELECT DISTINCT data_venda "
						+ "FROM taula_vendes ORDER by data_venda desc;", new DatesMapper());

		return dates;
	}

	public void setGestorVendes(GestorVendes gestorVendes) {
		this.gestorVendes = gestorVendes;
	}

	public GestorVendes getGestorVendes() {
		return gestorVendes;
	}
	
	private static class DatesMapper implements ParameterizedRowMapper<String> {

		public String mapRow(ResultSet rs, int numFila) throws SQLException {
			String dataSi = "";
			try {
				String data = rs.getString("data_venda");
				Date dataNo = new SimpleDateFormat("yyyy-MM-dd").parse(data);
				dataSi = new SimpleDateFormat("dd-MM-yyyy")
						.format(dataNo);
			} catch (ParseException e) {
				e.printStackTrace();
			}
			return dataSi;
		}

	}

	private static class VendesMapper implements ParameterizedRowMapper<Vendes> {

		public Vendes mapRow(ResultSet rs, int numFila) throws SQLException {
			Vendes ven = new Vendes();
			ven.setCodi(rs.getString("codi_producte"));
			try {
				String data = rs.getString("data_venda");
				Date dataNo = new SimpleDateFormat("yyyy-MM-dd").parse(data);
				String dataSi = new SimpleDateFormat("dd-MM-yyyy")
						.format(dataNo);
				ven.setData(dataSi);
			} catch (ParseException e) {
				e.printStackTrace();
			}
			ven.setnUsuari(rs.getString("nom_usuari"));
			ven.setQuantitat(rs.getInt("quantitat_venda"));
			ven.setPreu(rs.getFloat("preu"));
			return ven;
		}

	}

	private static class ProducteVenutMapper implements
			ParameterizedRowMapper<ProducteVenut> {

		public ProducteVenut mapRow(ResultSet rs, int numFila)
				throws SQLException {
			ProducteVenut pv = new ProducteVenut();
			pv.setCodi(rs.getString("codi_producte"));
			try {
				String data = rs.getString("data_venda");
				Date dataNo = new SimpleDateFormat("yyyy-MM-dd").parse(data);
				String dataSi = new SimpleDateFormat("dd-MM-yyyy")
						.format(dataNo);
				pv.setData(dataSi);
			} catch (ParseException e) {
				e.printStackTrace();
			}
			pv.setnUsuari(rs.getString("nom_usuari"));
			pv.setQuantitat(rs.getInt("quantitat_venda"));
			pv.setPreu(rs.getFloat("preu"));
			pv.setTitol(rs.getString("titol"));
			return pv;
		}

	}

}
